/**
 * laytpl - Node.js view engine
 */

'use strict';

var path = require('path');
var fs = require('fs');
var laytpl = require('./laytpl');

var dirname = path.dirname
var extname = path.extname
var join = path.join
var read = fs.readFileSync;


// 重新读取文件
laytpl.reinclude = laytpl.tool.reinclude = function(name, options) {
  var file = options.filename;
  if (!file) throw new Error('filename option is required for includes');

  var path = join(dirname(file), name);
  var ext = extname(name);

  if (!ext) path += '.html';
  return read(path, 'utf8');
};

// 嵌套模板 - 外部调用
laytpl.include = function(dir, options){
  var tpl = laytpl.reinclude(dir, options);
  return laytpl.getParse(tpl, options);
};

// 读取文件
laytpl.renderFile = function(file, options, fn){
  var tpl;
  if ('function' == typeof options) {
    fn = options, options = {};
  }
  options.filename = file;
  
  try {
    if(laytpl.options.cache){
     tpl = options.cache || (function(){
      if(cache[file]){
        return cache[file];
      } else {
        return cache[file] = read(file, 'utf8');
      }
     }());
    } else {
      tpl = read(file, 'utf8'); 
    }
  } catch(err) {
    fn(err);
    delete cache[file];
    return;
  };
  fn(null, laytpl.render(tpl, options));
};

// For Express.js
laytpl.__express = laytpl.renderFile;

module.exports = laytpl;