﻿/**
 * laytpl
 * 视图模板引擎
 */

'use strict';

var config = {
  open: '{{',
  close: '}}',
  cache: false,
  min: false
};

var tool = {
  exp: function(str){
    return new RegExp(str, 'g');
  },
  // 匹配满足规则内容
  query: function(type, _, __){
    var types = [
      '#([\\s\\S])+?'   // js 语句
      ,'([^{#}])*?' // 普通字段
    ][type || 0];
    return exp((_||'') + config.open + types + config.close + (__||''));
  },   
  escape: function(html){
    var exp = /[<"'>]|&(?=#[a-zA-Z0-9]+)/g;
    if(html === undefined || html === null) return '';
    
    html += '';
    if(!exp.test(html)) return html;

    return html.replace(/&(?!#?[a-zA-Z0-9]+;)/g, '&amp;')
    .replace(/</g, '&lt;').replace(/>/g, '&gt;')
    .replace(/'/g, '&#39;').replace(/"/g, '&quot;');
  },
  error: function(e){
    var error = 'Laytpl Error：';
    typeof console === 'object' && console.error(error + e);
    return error + e;
  }
};
 
var exp = tool.exp, cache = {}, LINE = 'var _________;';

// 解析模版
exports.parse = function(tpl, options){
  var that = this, file = options.filename;
  var jss = exp('^'+config.open+'#', ''), jsse = exp(config.close+'$', '');
  
  // 去除多余空格符
  if(config.min){
    tpl = tpl.replace(/\t/g, ' ').replace(/\s+/g, ' ');
  }

  tpl = tpl.replace(/[\r\n]+/g,  config.min ? '' : LINE)
  .replace(exp(config.open+'#'), config.open+'# ')
  .replace(exp(config.close+'}'), '} '+config.close).replace(/\\/g, '\\\\')
  
  // 不匹配指定区域的内容
  .replace(exp(config.open + '!(.+?)!' + config.close), function(str){
    str = str.replace(exp('^'+ config.open + '!'), '')
    .replace(exp('!'+ config.close), '')
    .replace(exp(config.open + '|' + config.close), function(tag){
      return tag.replace(/(.)/g, '\\$1')
    });
    return str
  })
  
  // 匹配 JS 语句
  .replace(/(?="|')/g, '\\').replace(tool.query(), function(str){
    str = str.replace(jss, '').replace(jsse, '');
    str = str.replace(exp(LINE), '');
    return '";' + str.replace(/\\(.)/g, '$1') + ';view+="';
  })
  
  // 匹配普通输出语句
  .replace(tool.query(1), function(str){
    var start = '"+laytpl.escape(';
    if(str.replace(/\s/g, '') === config.open + config.close){
      return '';
    }
    str = str.replace(exp(config.open+'|'+config.close), '');
    if(/^=/.test(str)){
      str = str.replace(/^=/, '');
    } else if(/^-/.test(str)){
      str = str.replace(/^-/, '');
      start = '"+(';
    }

    // include 语句
    if(/^\s*include/.test(str)){
      var match = str.match(/^\s*include\s+([\s\S]+?)\s*$/)||[];
      if(!match[1]) return '';
      return exports.parse(tool.reinclude(match[1], options), options);
    }

    return start + str.replace(/\\(.)/g, '$1') + ')+"';
  });
  return tpl;
};

// 获取编译后的模版
exports.getParse = function(tpl, options){
  var file = options.filename;

  tpl = '"use strict";var view = "' + exports.parse(tpl, options) + '";return view;';

  try {
    tpl = new Function('d, laytpl', tpl);
    if(config.cache) cache['tpl_' + file] = tpl;
    return tpl(options, tool);
  } catch(e) {
    delete cache['tpl_' + file];
    return tool.error(e);
  }
};

// 渲染
exports.render = function(tpl, options){
  var that = this, file = options.filename;
  if(config.cache && cache['tpl_' + file]){
    tpl = cache['tpl_' + file](options, tool.escape);
  } else {
    tpl = that.getParse(tpl, options);
  }
  return config.min ? tpl : tpl.replace(exp(LINE), '\n');
};

// 配置
exports.config = function(options){
  options = options || {};
  for(var i in options){
    config[i] = options[i];
  }
};

exports.options = config;
exports.tool = tool;


